@@ -22,7 +22,7 @@ from PySide6.QtWidgets import QWidget, QMenu, QAbstractItemDelegate, QComboBox,
 from PySide6.QtGui import (QStandardItem, QPalette, QContextMenuEvent, QDragMoveEvent, QDropEvent,
                            QDragEnterEvent, QKeyEvent, QStandardItem, QAction)
 
-from scrutiny.sdk import WatchableConfiguration, EmbeddedEnum
+from scrutiny.sdk import BriefWatchableConfiguration, EmbeddedEnum
 from scrutiny.gui.core.scrutiny_drag_data import ScrutinyDragData, WatchableListDescriptor
 from scrutiny.gui.core.watchable_registry import WatchableRegistry
 from scrutiny.gui.widgets.watchable_tree import (
@@ -387,13 +387,9 @@ class WatchComponentTreeModel(WatchableTreeModel):
         DATATYPE = 2
         ENUM = 3
 
-    class _Signals(QObject):
-        availability_changed = Signal(QModelIndex, bool, object)
-
     logger: logging.Logger
     _available_palette: QPalette
     _unavailable_palette: QPalette
-    _signals: _Signals
 
     def __init__(self,
                  parent: QWidget,
@@ -402,7 +398,6 @@ class WatchComponentTreeModel(WatchableTreeModel):
                  unavailable_palette: Optional[QPalette] = None) -> None:
         super().__init__(parent, watchable_registry)
         self.logger = logging.getLogger(self.__class__.__name__)
-        self._signals = self._Signals(self)
 
         if available_palette is not None:
             self._available_palette = available_palette
@@ -416,10 +411,6 @@ class WatchComponentTreeModel(WatchableTreeModel):
             self._unavailable_palette = QPalette()
             self._unavailable_palette.setCurrentColorGroup(QPalette.ColorGroup.Disabled)
 
-    @property
-    def signals(self) -> _Signals:
-        return self._signals
-
     def _assign_unique_watcher_id(self, item: WatchableStandardItem) -> None:
         watcher_id = global_i64_counter()
         item.setData(watcher_id, WATCHER_ID_ROLE)
@@ -436,7 +427,7 @@ class WatchComponentTreeModel(WatchableTreeModel):
     def itemFromIndex(self, index: Union[QModelIndex, QPersistentModelIndex]) -> BaseWatchableRegistryTreeStandardItem:
         return cast(BaseWatchableRegistryTreeStandardItem, super().itemFromIndex(index))
 
-    def get_watchable_extra_columns(self, fqn: str = "", watchable_config: Optional[WatchableConfiguration] = None) -> List[QStandardItem]:
+    def get_watchable_extra_columns(self, fqn: str = "", watchable_config: Optional[BriefWatchableConfiguration] = None) -> List[QStandardItem]:
         # We don't use watchable_config here even if we could.
         # We update the value/type when an item is available by calling update_row_state
         return [ValueStandardItem(), DataTypeStandardItem(), EnumNameStandardItem()]
@@ -616,8 +607,7 @@ class WatchComponentTreeModel(WatchableTreeModel):
                     first_col = cast(BaseWatchableRegistryTreeStandardItem, folder_row[0])
                     self.add_row_to_parent(parent, row_index, folder_row)
                     self.fill_from_index_recursive(first_col, parsed_fqn.watchable_type, parsed_fqn.path, keep_folder_fqn=False, editable=True)
-                    for item in self.get_all_watchable_items(parent):
-                        self.update_row_state(item)
+                    # No need to call update_row_state() here as the content comes directly from varlist so it's available.
 
                 elif node['type'] == 'watchable':
                     watchable_row = self.make_watchable_row(node['text'],
@@ -759,17 +749,11 @@ class WatchComponentTreeModel(WatchableTreeModel):
         """
         watchable_node = self._watchable_registry.get_watchable_node_fqn(watchable_item.fqn)
         watchable_config = watchable_node.configuration if watchable_node is not None else None
-        was_available = watchable_item.data(AVAILABLE_DATA_ROLE)
         if watchable_config is not None:
             self.set_available(watchable_item, watchable_config)
         else:
             self.set_unavailable(watchable_item)
 
-        is_available = watchable_item.data(AVAILABLE_DATA_ROLE)
-
-        if is_available != was_available:
-            self._signals.availability_changed.emit(watchable_item.index(), is_available, watchable_config)
-
     def set_unavailable(self, arg_item: WatchableStandardItem) -> None:
         """Make an item in the tree unavailable (grayed out)"""
         background_color = self._unavailable_palette.color(QPalette.ColorRole.Base)
@@ -789,7 +773,7 @@ class WatchComponentTreeModel(WatchableTreeModel):
                 elif isinstance(item, EnumNameStandardItem):
                     item.setText('')
 
-    def set_available(self, arg_item: WatchableStandardItem, watchable_config: WatchableConfiguration) -> None:
+    def set_available(self, arg_item: WatchableStandardItem, watchable_config: BriefWatchableConfiguration) -> None:
         """Make an item in the tree available (normal color)"""
         background_color = self._available_palette.color(QPalette.ColorRole.Base)
         forground_color = self._available_palette.color(QPalette.ColorRole.Text)
