@@ -85,17 +85,14 @@ class ServerRegistryBidirectionalMap:
         return len(self.s2r)
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class RegistryValueUpdate:
-    __slots__ = ('sdk_update', 'registry_id')
     sdk_update: ValueUpdate
     registry_id: int
 
 
-@dataclass
+@dataclass(slots=True)
 class ParsedFullyQualifiedName:
-    __slots__ = ('watchable_type', 'path')
-
     watchable_type: sdk.WatchableType
     path: str
 
@@ -122,39 +119,35 @@ TYPESTR_MAP_WT2S: Dict[sdk.WatchableType, str] = {v: k for k, v in TYPESTR_MAP_S
 
 
 WatcherValueUpdateCallback = Callable[[WatcherIdType, List[RegistryValueUpdate]], None]
-UnwatchCallback = Callable[[WatcherIdType, str, sdk.WatchableConfiguration, int], None]
-GlobalWatchCallback = Callable[[WatcherIdType, str, sdk.WatchableConfiguration, int], None]
-GlobalUnwatchCallback = Callable[[WatcherIdType, str, sdk.WatchableConfiguration, int], None]
+UnwatchCallback = Callable[[WatcherIdType, str, sdk.BriefWatchableConfiguration, int], None]
+GlobalWatchCallback = Callable[[WatcherIdType, str, sdk.BriefWatchableConfiguration, int], None]
+GlobalUnwatchCallback = Callable[[WatcherIdType, str, sdk.BriefWatchableConfiguration, int], None]
 
 
-@dataclass(init=False)
+@dataclass(init=False, slots=True)
 class WatchableRegistryEntryNode:
     """Leaf node in the tree that is a single watchable"""
-
-    __slots__ = ('configuration', 'server_path', 'registry_id', '_watcher_count')
-
-    configuration: sdk.WatchableConfiguration
+    configuration: sdk.BriefWatchableConfiguration
     server_path: str
     registry_id: int
     _watcher_count: int
 
-    def __init__(self, registry: "WatchableRegistry", server_path: str, config: sdk.WatchableConfiguration) -> None:
+    def __init__(self, registry: "WatchableRegistry", server_path: str, config: sdk.BriefWatchableConfiguration) -> None:
         self.server_path = server_path
         self.configuration = config
         self._watcher_count = 0
         self.registry_id = registry._make_node_id()
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class WatchableRegistryIntermediateNode:
     """An intermediate node that contains watchable and other subnodes"""
-    __slots__ = ['watchables', 'subtree']
 
     watchables: Dict[str, WatchableRegistryEntryNode]
     subtree: List[str]
 
 
-@dataclass(init=False)
+@dataclass(init=False, slots=True)
 class Watcher:
     watcher_id: WatcherIdType
     value_update_callback: WatcherValueUpdateCallback
@@ -184,7 +177,7 @@ class WatchableRegistry:
     """Contains a copy of the watchable list available on the server side
     Act as a relay to dispatch value update event to the internal widgets"""
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class Statistics:
         """(Immutable struct) Internal metrics for debugging and diagnostics"""
         watched_entries_count: int
@@ -251,7 +244,7 @@ class WatchableRegistry:
         return v
 
     @enforce_thread(QT_THREAD_NAME)
-    def _add_watchable(self, path: str, config: sdk.WatchableConfiguration) -> None:
+    def _add_watchable(self, path: str, config: sdk.BriefWatchableConfiguration) -> None:
         """Adds a single watchable to the tree storage
 
         :param path: Path to add the node to
@@ -661,7 +654,7 @@ class WatchableRegistry:
         return isinstance(node, WatchableRegistryEntryNode)
 
     @enforce_thread(QT_THREAD_NAME)
-    def write_content(self, data: Dict[sdk.WatchableType, Dict[str, sdk.WatchableConfiguration]]) -> None:
+    def write_content(self, data: Dict[sdk.WatchableType, Dict[str, sdk.BriefWatchableConfiguration]]) -> None:
         """Write content of the given types.
         Triggers ``changed``.  May trigger ``filled`` if all types have data after calling this function.
 
