@@ -32,8 +32,12 @@ __all__ = [
     'DeviceLinkInfo',
     'ServerInfo',
     'UserCommandResponse',
-    'WatchableConfiguration',
-    'WatchableConfigurationWithServerID',
+    'BriefWatchableConfiguration',
+    'BaseDetailedWatchableConfiguration',
+    'DetailedVarWatchableConfiguration',
+    'DetailedAliasWatchableConfiguration',
+    'DetailedRPVWatchableConfiguration',
+    'DetailedWatchableConfiguration',
     'VariableFactoryInterface',
     'WatchableListContentPart',
     'DataloggingEncoding',
@@ -49,6 +53,7 @@ import enum
 import math
 import itertools
 from dataclasses import dataclass
+from scrutiny.sdk import exceptions as sdk_exceptions
 from scrutiny.core.basic_types import MemoryRegion, EmbeddedDataType, WatchableType
 from scrutiny.core.embedded_enum import EmbeddedEnum
 from scrutiny.core import path_tools
@@ -209,7 +214,7 @@ class SamplingRate:
     """Name for display only"""
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class FixedFreqSamplingRate(SamplingRate):
     """(Immutable struct) Represent a fixed frequency sampling rate supported by the device"""
 
@@ -222,7 +227,7 @@ class FixedFreqSamplingRate(SamplingRate):
         validation.assert_type(self.frequency, 'frequency', float)
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class VariableFreqSamplingRate(SamplingRate):
     """(Immutable struct) Represent a variable frequency sampling rate supported by the device. Has no known frequency"""
 
@@ -325,7 +330,7 @@ class BaseLinkConfig(abc.ABC):
         raise NotImplementedError("Abstract class")
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class NoneLinkConfig(BaseLinkConfig):
     """(Immutable struct) An Empty object acting as configuration structure for a device link of type :attr:`NONE<scrutiny.sdk.DeviceLinkType.NONE>`
     Exists only to differentiate ``None`` (data not available) from :class:`NoneLinkConfig<scrutiny.sdk.NoneLinkConfig>` (data available - no link configured)
@@ -335,7 +340,7 @@ class NoneLinkConfig(BaseLinkConfig):
         return {}
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class UDPLinkConfig(BaseLinkConfig):
     """(Immutable struct) The configuration structure for a device link of type :attr:`UDP<scrutiny.sdk.DeviceLinkType.UDP>`"""
 
@@ -354,7 +359,7 @@ class UDPLinkConfig(BaseLinkConfig):
         }
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class TCPLinkConfig(BaseLinkConfig):
     """(Immutable struct) The configuration structure for a device link of type :attr:`TCP<scrutiny.sdk.DeviceLinkType.TCP>`"""
 
@@ -374,7 +379,7 @@ class TCPLinkConfig(BaseLinkConfig):
         }
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class SerialLinkConfig(BaseLinkConfig):
     """(Immutable struct) The configuration structure for a device link of type :attr:`Serial<scrutiny.sdk.DeviceLinkType.Serial>`"""
 
@@ -479,7 +484,7 @@ class SerialLinkConfig(BaseLinkConfig):
         }
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class RTTLinkConfig(BaseLinkConfig):
     """(Immutable struct) The configuration structure for a device link of type :attr:`RTT<scrutiny.sdk.DeviceLinkType.RTT>`"""
 
@@ -538,7 +543,7 @@ class RTTLinkConfig(BaseLinkConfig):
 # region CAN bus
 
 
-@dataclass(frozen=True)
+@dataclass(frozen=True, slots=True)
 class CANLinkConfig(BaseLinkConfig):
     """(Immutable struct) The configuration structure for a device link of type :attr:`CAN<scrutiny.sdk.DeviceLinkType.CAN>`"""
 
@@ -556,7 +561,7 @@ class CANLinkConfig(BaseLinkConfig):
         ETAS = 4
         """Use ETAS hardware through the ETAS DLL."""
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class SocketCANConfig:
         """
         (Immutable struct) A SocketCAN specific configuration used when 
@@ -575,7 +580,7 @@ class CANLinkConfig(BaseLinkConfig):
                 'channel': self.channel
             }
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class VectorConfig:
         """
         (Immutable struct) A Vector specific configuration used when 
@@ -606,7 +611,7 @@ class CANLinkConfig(BaseLinkConfig):
                 'data_bitrate': self.data_bitrate
             }
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class KVaserConfig:
         """
         (Immutable struct) A KVaser specific configuration used when 
@@ -638,7 +643,7 @@ class CANLinkConfig(BaseLinkConfig):
                 'fd_non_iso': self.fd_non_iso
             }
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class PCANConfig:
         """
         (Immutable struct) A PCAN specific configuration used when 
@@ -662,7 +667,7 @@ class CANLinkConfig(BaseLinkConfig):
                 'bitrate': self.bitrate
             }
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class ETASConfig:
         """
         (Immutable struct) A ETAS specific configuration used when 
@@ -807,7 +812,7 @@ class UserCommandResponse:
 
 
 @dataclass(frozen=True, slots=True)
-class WatchableConfiguration:
+class BriefWatchableConfiguration:
     """(Immutable struct) Represents a watchable available in the server datastore"""
 
     watchable_type: WatchableType
@@ -821,16 +826,114 @@ class WatchableConfiguration:
 
 
 @dataclass(frozen=True, slots=True)
-class WatchableConfigurationWithServerID(WatchableConfiguration):
-    """(Immutable struct) Represents a watchable available in the server datastore with a server_id available"""
-
+class BaseDetailedWatchableConfiguration(BriefWatchableConfiguration):
+    """(Immutable struct) Base class to be extended by: 
+        :class:`DetailedVarWatchableConfiguration<scrutiny.sdk.DetailedVarWatchableConfiguration>`,
+        :class:`DetailedAliasWatchableConfiguration<scrutiny.sdk.DetailedAliasWatchableConfiguration>`,
+        :class:`DetailedRPVWatchableConfiguration<scrutiny.sdk.DetailedRPVWatchableConfiguration>` 
+    """
     server_id: str
     """The unique ID assigned to that watchable item by the server"""
 
+    server_path: str
+    """The path assigned by the server to this watchable"""
+
+    @property
+    def name(self) -> str:
+        """Returns the watchable name, e.g. the basename in the server_path. 
+        If the server_path is /aaa/bbb/ccc, then this returns "ccc" """
+        return path_tools.make_segments(self.server_path)[-1]
+
+    def has_enum(self) -> bool:
+        """Tells if the watchable has an enum associated with it"""
+        return self.enum is not None
+
+    def get_enum(self) -> EmbeddedEnum:
+        """ Returns the enum associated with this watchable
+
+        :raises BadEnumError: If the watchable has no enum assigned
+        """
+        self._assert_has_enum()
+        assert self.enum is not None
+        return self.enum.copy()
+
+    def _assert_has_enum(self) -> None:
+        if not self.has_enum():
+            raise sdk_exceptions.BadEnumError(f"Watchable {self.name} has no enum defined")
+
+    def parse_enum_val(self, val: str) -> int:
+        """Converts an enum value name (string) to the underlying integer value
+
+        :param val: The enumerator name to convert
+
+        :raises BadEnumError: If the watchable has no enum assigned or the given value is not a valid enumerator
+        :raise TypeError: Given parameter not of the expected type
+        """
+        validation.assert_type(val, 'val', str)
+
+        self._assert_has_enum()
+        assert self.enum is not None
+        if not self.enum.has_value(val):
+            raise sdk_exceptions.BadEnumError(f"Value {val} is not a valid value for enum {self.enum.name}")
+
+        return self.enum.get_value(val)
+
+
+@dataclass(frozen=True, slots=True)
+class DetailedVarWatchableConfiguration(BaseDetailedWatchableConfiguration):
+    """Structure containing the metadata of a watchable of type :attr:`Variable<scrutiny.sdk.WatchableType.Variable>`"""
+
+    address: int
+    """Absolute memory address of the variable in the firmware"""
+
+    bitoffset: Optional[int]
+    """For bitfield, the startbit of the value. ``None`` if not a bitfield"""
+
+    bitsize: Optional[int]
+    """For bitfield, the size in bits of the value. ``None`` if not a bitfield"""
+
+
+@dataclass(frozen=True, slots=True)
+class DetailedAliasWatchableConfiguration(BaseDetailedWatchableConfiguration):
+    """Structure containing the metadata of a watchable of type :attr:`Alias<scrutiny.sdk.WatchableType.Alias>`"""
+
+    target: str
+    """The server path to the reference watchable (variable or alias)"""
+
+    target_type: WatchableType
+    """The type of watchable referenced by this alias"""
+
+    gain: Optional[float]
+    """An optional gain. Scale the value when read or writen. Does not apply when ``None``"""
+
+    offset: Optional[float]
+    """An optional offset. Add a bias to the value when read or writen. Does not apply when ``None``"""
+
+    min: Optional[float]
+    """An optional lower bound that applies only when written. Does not apply when ``None``"""
+
+    max: Optional[float]
+    """An optional upper bound that applies only when written. Does not apply when ``None``"""
+
+
+@dataclass(frozen=True, slots=True)
+class DetailedRPVWatchableConfiguration(BaseDetailedWatchableConfiguration):
+    """Structure containing the metadata of a watchable of type :attr:`RuntimePublishedValue<scrutiny.sdk.WatchableType.RuntimePublishedValue>`"""
+
+    rpvid: int
+    """The numerical 16bits ID of this RPV"""
+
+
+DetailedWatchableConfiguration: TypeAlias = Union[
+    DetailedVarWatchableConfiguration,
+    DetailedAliasWatchableConfiguration,
+    DetailedRPVWatchableConfiguration
+]
+
 
 @dataclass(frozen=True, slots=True)
 class VariableFactoryInterface:
-    """(Immutable struct) Represent a VariableFactory in the server. Can be sued to generate locally
+    """(Immutable struct) Represent a VariableFactory in the server. Can be used to generate locally
     all the variables that the server can instantiate from this factory.
     Mainly used to create every array elements from a set of dimensions
     """
@@ -849,7 +952,7 @@ class VariableFactoryInterface:
 
     def count_possible_paths(self) -> int:
         """Returns how many path his factory can generate"""
-        if not self.is_valid():
+        if not self._is_valid():
             return 0
 
         total = 1
@@ -857,8 +960,8 @@ class VariableFactoryInterface:
             total *= math.prod(dims)
         return total
 
-    def is_valid(self) -> bool:
-        """Tells if this VariableFactory as valid parameters"""
+    def _is_valid(self) -> bool:
+        """Tells if this VariableFactory has valid content. Can be used for sanity check."""
         segments = path_tools.make_segments(self.access_path)
         for path, dims in self.array_dims.items():
             array_segments = path_tools.make_segments(path)
@@ -877,9 +980,9 @@ class VariableFactoryInterface:
 
         return True
 
-    def iterate_possible_paths(self) -> Generator[Tuple[str, WatchableConfiguration], None, None]:
+    def iterate_possible_paths(self) -> Generator[Tuple[str, BriefWatchableConfiguration], None, None]:
         """Return all the variable display path that can be generated by this factory"""
-        if not self.is_valid():
+        if not self._is_valid():
             return
 
         segments = path_tools.make_segments(self.access_path)
@@ -901,7 +1004,7 @@ class VariableFactoryInterface:
                 segments_copy[segments_position_lookup[i]] += ''.join([f'[{p}]' for p in segment_pos])
 
             path = path_tools.join_segments(segments_copy)
-            watchable_def = WatchableConfiguration(
+            watchable_def = BriefWatchableConfiguration(
                 datatype=self.datatype,
                 watchable_type=WatchableType.Variable,
                 enum=self.enum
@@ -914,9 +1017,9 @@ class WatchableListContentPart:
     """A partial dataset received from the server when downloading the watchable list"""
 
     def __init__(self,
-                 var: Optional[Dict[str, WatchableConfiguration]] = None,
-                 alias: Optional[Dict[str, WatchableConfiguration]] = None,
-                 rpv: Optional[Dict[str, WatchableConfiguration]] = None,
+                 var: Optional[Dict[str, BriefWatchableConfiguration]] = None,
+                 alias: Optional[Dict[str, BriefWatchableConfiguration]] = None,
+                 rpv: Optional[Dict[str, BriefWatchableConfiguration]] = None,
                  var_factory: Optional[Dict[str, VariableFactoryInterface]] = None
                  ) -> None:
         self.var = {} if var is None else var
@@ -924,13 +1027,13 @@ class WatchableListContentPart:
         self.rpv = {} if rpv is None else rpv
         self.var_factory = {} if var_factory is None else var_factory
 
-    var: Dict[str, WatchableConfiguration]
+    var: Dict[str, BriefWatchableConfiguration]
     """The variables in the server datastore"""
 
-    alias: Dict[str, WatchableConfiguration]
+    alias: Dict[str, BriefWatchableConfiguration]
     """The aliases in the server datastore """
 
-    rpv: Dict[str, WatchableConfiguration]
+    rpv: Dict[str, BriefWatchableConfiguration]
     """The runtime Published values in the server datastore """
 
     var_factory: Dict[str, VariableFactoryInterface]
