@@ -45,7 +45,7 @@ class FsmState(enum.Enum):
     SHUTDOWN_CLEAR_PENDING_REQUEST = enum.auto()
 
 
-@dataclass
+@dataclass(slots=True)
 class DeviceSideAcquisitionRequest:
     api_request: api_datalogging.AcquisitionRequest
     device_config: device_datalogging.Configuration
@@ -54,15 +54,24 @@ class DeviceSideAcquisitionRequest:
 
 
 class DataloggingManager:
-    datastore: Datastore    # Reference to the server datastore
-    device_handler: DeviceHandler   # Reference to the device handler
-    acquisition_request_queue: "queue.Queue[DeviceSideAcquisitionRequest]"  # The queue in which acquisition requests are put in by the API
-    active_request: Optional[DeviceSideAcquisitionRequest]  # The acquisition request being actively processed. None when no request is processed
-    logger: logging.Logger  # Logger
+    datastore: Datastore
+    """Reference to the server datastore"""
+    device_handler: DeviceHandler
+    """Reference to the device handler"""
+    acquisition_request_queue: "queue.Queue[DeviceSideAcquisitionRequest]"
+    """The queue in which acquisition requests are put in by the API"""
+    active_request: Optional[DeviceSideAcquisitionRequest]
+    """The acquisition request being actively processed. None when no request is processed"""
+    logger: logging.Logger
+    """The class logger"""
     state: FsmState
+    """A state machine state"""
     previous_state: FsmState
+    """The state machine previous state"""
     _previous_datalogging_state_and_completion: Tuple[api_datalogging.DataloggingState, Optional[float]]
+    """Keep track of the state of the datalogger from the previous process() call"""
     _datalogging_state_changed_callbacks: List[DataloggingStateChangedCallback]
+    """List of callbacks whenever the datalogging state changes"""
 
     TIME_PRECISION_DIGIT = 9    # Device precision is 1e-7. 9 digits is more than enough.
 
@@ -141,7 +150,7 @@ class DataloggingManager:
 
         return out_data
 
-    def acquisition_complete_callback(self, success: bool, detail_msg: str, data: Optional[List[List[bytes]]], metadata: Optional[device_datalogging.AcquisitionMetadata]) -> None:
+    def _acquisition_complete_callback(self, success: bool, detail_msg: str, data: Optional[List[List[bytes]]], metadata: Optional[device_datalogging.AcquisitionMetadata]) -> None:
         """Callback called by the device handler when the acquisition finally gets triggered and data has finished downloaded."""
         if self.active_request is None:
             self.logger.error("Received acquisition data but was not expecting it. No active acquisition request")
@@ -190,7 +199,7 @@ class DataloggingManager:
 
                 # Now converts binary data into meaningful value using the datastore entries and add to acquisition object
                 for signal in self.active_request.api_request.signals:
-                    parsed_data = self.read_active_request_data_from_raw_data(signal, data)  # Parse binary data
+                    parsed_data = self._read_active_request_data_from_raw_data(signal, data)  # Parse binary data
                     ds = DataSeries(
                         data=parsed_data,
                         logged_watchable=LoggedWatchable(
@@ -228,7 +237,7 @@ class DataloggingManager:
                     if xaxis_signal.name is None:
                         xaxis_signal = api_datalogging.SignalDefinition(name='X-Axis', entry=xaxis_signal.entry)
                     assert xaxis_signal.name is not None
-                    parsed_data = self.read_active_request_data_from_raw_data(xaxis_signal, data)
+                    parsed_data = self._read_active_request_data_from_raw_data(xaxis_signal, data)
                     xaxis.set_data(parsed_data)
                     xaxis.name = xaxis_signal.name
                     xaxis.logged_watchable = LoggedWatchable(
@@ -271,7 +280,7 @@ class DataloggingManager:
         if err:
             raise err
 
-    def read_active_request_data_from_raw_data(self, signal: api_datalogging.SignalDefinition, data: List[List[bytes]]) -> List[float]:
+    def _read_active_request_data_from_raw_data(self, signal: api_datalogging.SignalDefinition, data: List[List[bytes]]) -> List[float]:
         """Converts a List of binary blocks into a list of numeric values (64 bits float) using the datastore definitions generated by the debug symbols."""
         assert self.active_request is not None
         loggable_id = self.active_request.entry_signal_map[signal.entry]
@@ -333,7 +342,7 @@ class DataloggingManager:
                             self.device_handler.request_datalogging_acquisition(
                                 loop_id=self.active_request.api_request.rate_identifier,
                                 config=self.active_request.device_config,
-                                callback=self.acquisition_complete_callback
+                                callback=self._acquisition_complete_callback
                             )
                         else:
                             # Cause of not ready:
@@ -438,7 +447,7 @@ class DataloggingManager:
         return device_cond
 
     @classmethod
-    def make_device_config_from_request(self,
+    def make_device_config_from_request(cls,
                                         request: api_datalogging.AcquisitionRequest
                                         ) -> Tuple[device_datalogging.Configuration, Dict[DatastoreEntry, int]]:
         """Converts a Configuration from the API format to the device format"""
@@ -448,7 +457,7 @@ class DataloggingManager:
         config.timeout = request.timeout
         config.probe_location = request.probe_location
         config.trigger_hold_time = request.trigger_hold_time
-        config.trigger_condition = self.api_trigger_condition_to_device_trigger_condition(request.trigger_condition)
+        config.trigger_condition = cls.api_trigger_condition_to_device_trigger_condition(request.trigger_condition)
 
         entry2signal_map: Dict[DatastoreEntry, int] = {}
 
@@ -468,7 +477,7 @@ class DataloggingManager:
                 entry_to_log = signal.entry
 
             if entry_to_log not in entry2signal_map:
-                config.add_signal(self.make_signal_from_watchable(entry_to_log))
+                config.add_signal(cls.make_signal_from_watchable(entry_to_log))
                 signal_index = len(config.get_signals()) - 1
             else:
                 signal_index = entry2signal_map[entry_to_log]
@@ -583,6 +592,8 @@ class DataloggingManager:
         return output
 
     def get_datalogging_state(self) -> Tuple[api_datalogging.DataloggingState, Optional[float]]:
+        """Return the state of the datalogging service offered by the server.
+        Translated from the device datalogger state."""
         if self.state != FsmState.DEVICE_CONNECTED_WITH_DATALOGGING:
             return (api_datalogging.DataloggingState.NA, None)
 
