@@ -24,7 +24,6 @@ import logging
 import binascii
 import time
 from enum import Enum, auto
-import traceback
 from uuid import uuid4
 import math
 import threading
@@ -36,6 +35,7 @@ from scrutiny.server.protocol import *
 from scrutiny.server.protocol.comm_handler import CommHandler
 from scrutiny.server.protocol.commands import DummyCommand
 from scrutiny.server.device.request_dispatcher import RequestDispatcher, RequestRecord
+from scrutiny.server.device.submodules.base_device_handler_submodule import BaseDeviceHandlerSubmodule
 from scrutiny.server.device.submodules.device_searcher import DeviceSearcher
 from scrutiny.server.device.submodules.heartbeat_generator import HeartbeatGenerator
 from scrutiny.server.device.submodules.info_poller import InfoPoller
@@ -52,6 +52,7 @@ from scrutiny.server.device.links import AbstractLink, LinkConfig
 from scrutiny.core.firmware_id import PLACEHOLDER as DEFAULT_FIRMWARE_ID
 from scrutiny.core.basic_types import WatchableType
 from scrutiny import tools
+from scrutiny.tools import validation
 
 from scrutiny.tools.typing import *
 
@@ -110,44 +111,76 @@ class DeviceHandler:
     and will run read/write request on the device to keep the datastore in sync with the embedded device.
     """
 
-    @dataclass(frozen=True)
+    @dataclass(frozen=True, slots=True)
     class Statistics:
         comm_handler: CommHandler.Statistics
         device_session_count: int
 
     logger: logging.Logger
-    config: DeviceHandlerConfig             # The configuration coming from the user
-    datastore: Datastore    # A reference to the main Datastore
-    dispatcher: RequestDispatcher           # The arbiter that receive all request to be sent and decides which one to send (uses a priority queue)
-    device_searcher: DeviceSearcher         # Component of the DeviceHandler that search for a new device on the communication link
-    session_initializer: SessionInitializer  # Component of the DeviceHandler that try to establish a connection to a found device
-    heartbeat_generator: HeartbeatGenerator  # Component of the DeviceHandler that periodically enqueue heartbeat request to keep a session active
-    memory_reader: MemoryReader     # Component of the DeviceHandler that polls the device memory to keep the datastore watched entries in sync with the device
-    memory_writer: MemoryWriter     # Component of the DeviceHandler that fulfill writes requests on datastore entries
-    info_poller: InfoPoller         # Component of the DeviceHandler that will gather all device parameters after a connection is established
-    datalogging_poller: DataloggingPoller  # Component of the DeviceHandler that will interact with the device datalogging feature
-    comm_handler: CommHandler       # Layer that handle the communication with the device. Converts Requests to bytes and bytes to response. Also tells if a request timed out
-    protocol: Protocol      # The communication protocol. Encode and decodes request/response payload to meaning ful data
-    device_info: Optional[DeviceInfo]   # All the information about the device gathered by the InfoPoller
-    comm_broken: bool   # Flag indicating that we lost connection with the device. Resets the communication and the state machine.
-    device_id: Optional[str]    # Firmware ID of the device on which we are connected.
-    operating_mode: "DeviceHandler.OperatingMode"   # Operating mode - Normal or special modes for unit testing
-    connected: bool  # True when a connection to a device has been made
-    fsm_state: "DeviceHandler.FsmState"         # The internal state machine state
-    last_fsm_state: "DeviceHandler.FsmState"    # The state machine state at the previous execution cycle
-    active_request_record: Optional[RequestRecord]  # Request to the device on which we are waiting for
-    device_session_id: Optional[int]       # The session ID given by the device upon connection
-    disconnection_requested: bool   # The external world requested that we disconnect from the device
-    disconnect_callback: Optional[DisconnectCallback]   # Callback called upon disconnection
-    disconnect_complete: bool   # Flags indicating that a disconnect request has been completed
-    comm_broken_count: int      # Counter keeping track of how many time we had communication issues
-    fully_connected_ready: bool  # Indicates that the DeviceHandler is connected to a device and the initialization phase is correctly completed
-    wait_clean_state_timestamp: float  # Timestamp to detect subcomponents that might be stuck
-    server_session_id: Optional[str]    # A session id generated by the DeviceHandler upon connection
-    device_state_changed_callbacks: List[DeviceStateChangedCallback]    # Calback called when the state machine changes state
-    expect_no_timeout: bool  # Flag that makes communicaiton timeout a critical error. Used for unit testing.
-    device_session_count: int    # Counter to keep track of how many device the server has connected to
+    config: DeviceHandlerConfig
+    """The configuration coming from the user"""
+    datastore: Datastore
+    """A reference to the main Datastore"""
+    dispatcher: RequestDispatcher
+    """The arbiter that receive all request to be sent and decides which one to send (uses a priority queue)"""
+    device_searcher: DeviceSearcher
+    """Component of the DeviceHandler that search for a new device on the communication link"""
+    session_initializer: SessionInitializer
+    """Component of the DeviceHandler that try to establish a connection to a found device"""
+    heartbeat_generator: HeartbeatGenerator
+    """Component of the DeviceHandler that periodically enqueue heartbeat request to keep a session active"""
+    memory_reader: MemoryReader
+    """Component of the DeviceHandler that polls the device memory to keep the datastore watched entries in sync with the device"""
+    memory_writer: MemoryWriter
+    """Component of the DeviceHandler that fulfill writes requests on datastore entries"""
+    info_poller: InfoPoller
+    """Component of the DeviceHandler that will gather all device parameters after a connection is established"""
+    datalogging_poller: DataloggingPoller
+    """Component of the DeviceHandler that will interact with the device datalogging feature"""
+    comm_handler: CommHandler
+    """Layer that handle the communication with the device. Converts Requests to bytes and bytes to response. Also tells if a request timed out"""
+    protocol: Protocol
+    """The communication protocol. Encode and decodes request/response payload to meaning ful data"""
+    device_info: Optional[DeviceInfo]
+    """All the information about the device gathered by the InfoPoller"""
+    comm_broken: bool
+    """Flag indicating that we lost connection with the device. Resets the communication and the state machine."""
+    device_id: Optional[str]
+    """Firmware ID of the device on which we are connected."""
+    operating_mode: "DeviceHandler.OperatingMode"
+    """Operating mode - Normal or special modes for unit testing"""
+    connected: bool
+    """True when a connection to a device has been made"""
+    fsm_state: "DeviceHandler.FsmState"
+    """The internal state machine state"""
+    last_fsm_state: "DeviceHandler.FsmState"
+    """The state machine state at the previous execution cycle"""
+    active_request_record: Optional[RequestRecord]
+    """Request to the device on which we are waiting for"""
+    device_session_id: Optional[int]
+    """The session ID given by the device upon connection"""
+    disconnection_requested: bool
+    """The external world requested that we disconnect from the device"""
+    disconnect_callback: Optional[DisconnectCallback]
+    """Callback called upon disconnection"""
+    disconnect_complete: bool
+    """Flags indicating that a disconnect request has been completed"""
+    comm_broken_count: int
+    """Counter keeping track of how many time we had communication issues"""
+    fully_connected_ready: bool
+    """Indicates that the DeviceHandler is connected to a device and the initialization phase is correctly completed"""
+    wait_clean_state_timestamp: float
+    """Timestamp to detect subcomponents that might be stuck"""
+    server_session_id: Optional[str]
+    """A session id generated by the DeviceHandler upon connection"""
+    device_state_changed_callbacks: List[DeviceStateChangedCallback]
+    """Callback called when the state machine changes state"""
+    expect_no_timeout: bool
+    """Flag that makes communication timeout a critical error. Used for unit testing."""
+    device_session_count: int
+    """Counter to keep track of how many device the server has connected to"""
     _demo_device: Optional[DemoDevice]
+    """A fake device that runs in a separate thread used for demonstrating the GUI or the SDK"""
 
     DEFAULT_PARAMS: DeviceHandlerConfig = {
         'response_timeout': 1.0,    # If a response take more than this delay to be received after a request is sent, drop the response.
@@ -213,6 +246,17 @@ class DeviceHandler:
         update_dict_recursive(self.config, config)  # type: ignore
         self.datastore = datastore
         self.dispatcher = RequestDispatcher()
+        try:
+            validation.assert_dict_key(self.config, 'default_protocol_version', str)
+            validation.assert_dict_key(self.config, 'response_timeout', (int, float))
+            validation.assert_dict_key(self.config, 'heartbeat_timeout', (int, float))
+            validation.assert_dict_key(self.config, 'default_address_size', int)
+            validation.assert_dict_key(self.config, 'max_request_size', int)
+            validation.assert_dict_key(self.config, 'max_response_size', int)
+            validation.assert_dict_key(self.config, 'max_bitrate_bps', (int, float))
+        except Exception as e:
+            raise ValueError(f"Error in device handler configuration. {e}") from e
+
         (major, minor) = self.config['default_protocol_version'].split('.')
         self.protocol = Protocol(int(major), int(minor))
         self.device_searcher = DeviceSearcher(self.protocol, self.dispatcher, priority=self.RequestPriority.Discover)
@@ -488,6 +532,26 @@ class DeviceHandler:
         """Returns what type of link is used to communicate with the device (serial, UDP, CanBus, etc)"""
         return self.comm_handler.get_link_type()
 
+    def need_keep_alive(self) -> bool:
+        """Inform the server layer that a call to process() will make meaningful work, therefore, avoid sleeping"""
+        if self.active_request_record is not None:  # waiting on data
+            return False
+
+        submodule: BaseDeviceHandlerSubmodule
+        for submodule in [
+            self.device_searcher,
+            self.session_initializer,
+            self.heartbeat_generator,
+            self.memory_reader,
+            self.memory_writer,
+            self.info_poller,
+            self.datalogging_poller,
+        ]:
+            if submodule.would_send_data():
+                return True
+
+        return False
+
     # Set communication state to a fresh start.
     def reset_comm(self) -> None:
         """Reset the communication with the device. Reset all state machines, clear pending requests, reset internal status"""
@@ -842,7 +906,7 @@ class DeviceHandler:
                     )
 
             if self.disconnect_complete:
-                next_state != self.FsmState.DISCONNECTING
+                next_state = self.FsmState.INIT
 
         else:
             raise Exception('Unknown FSM state : %s' % self.fsm_state)
