@@ -17,6 +17,7 @@ __all__ = [
 import logging
 import queue
 
+from scrutiny.server.device.submodules.base_device_handler_submodule import BaseDeviceHandlerSubmodule
 from scrutiny.server.timebase import server_timebase
 from scrutiny.server.protocol import *
 import scrutiny.server.protocol.commands as cmd
@@ -34,12 +35,21 @@ RawMemoryWriteRequestCompletionCallback = Callable[["RawMemoryWriteRequest", boo
 
 
 class RawMemoryWriteRequest:
+    """An internal structure used to define a raw memory write, meaning a read not tied to a datastore entry. 
+    Just the user that wants to dump the content"""
+
     address: int
+    """Address to read"""
     data: bytes
+    """Size of to read"""
     completed: bool
+    """Indicates if the request is completed (failed or successful)"""
     success: bool
+    """Indicates if the request is completed and successful"""
     completion_callback: Optional[RawMemoryWriteRequestCompletionCallback]
+    """Callback to call upon request completion"""
     completion_server_time_us: Optional[float]
+    """A timestamp of when the request completed"""
 
     def __init__(self, address: int, data: bytes, callback: Optional[RawMemoryWriteRequestCompletionCallback] = None):
         self.address = address
@@ -57,33 +67,53 @@ class RawMemoryWriteRequest:
             self.completion_callback(self, success, self.completion_server_time_us, failure_reason)
 
 
-class MemoryWriter:
+class MemoryWriter(BaseDeviceHandlerSubmodule):
+    """Class that request the device for its memory content or RPV to me modified according to 
+    either a datastore entry value or a raw value requested by the user
+    It treats Variable and RuntimePublishedValues differently as they use a different protocol command.
+    """
 
     DEFAULT_MAX_REQUEST_PAYLOAD_SIZE: int = 1024
     DEFAULT_MAX_RESPONSE_PAYLOAD_SIZE: int = 1024
 
     logger: logging.Logger
-    dispatcher: RequestDispatcher       # We put the request in here, and we know they'll go out
-    protocol: Protocol                  # The actual protocol. Used to build the request payloads
-    request_priority: int               # Our dispatcher priority
-    datastore: Datastore    # The datastore the look for entries to update
-    stop_requested: bool    # Requested to stop polling
-    pending_request: Optional[Request]   # The request presently being sent
-    started: bool           # Indicate if enabled or not
-    max_request_payload_size: int   # Maximum size for a request payload gotten from the InfoPoller
-    max_response_payload_size: int  # Maximum size for a response payload gotten from the InfoPoller
-    forbidden_regions: List[MemoryRegion]    # List of memory regions to avoid. Gotten from InfoPoller
-    readonly_regions: List[MemoryRegion]     # List of memory region that can only be read. Gotten from InfoPoller
-    memory_write_allowed: bool  # Indicates if writing to memory is allowed.
-
-    entry_being_updated: Optional[DatastoreEntry]   # The datastore entry updated by the actual pending request. None if no request is pending
-    # Update request attached to the entry being updated. It's what'S coming from the API
+    """The class logger"""
+    dispatcher: RequestDispatcher
+    """We put the request in here, and we know they'll go out"""
+    protocol: Protocol
+    """The actual protocol. Used to build the request payloads"""
+    request_priority: int
+    """Our dispatcher priority"""
+    datastore: Datastore
+    """The datastore the look for entries to update"""
+    stop_requested: bool
+    """Requested to stop polling"""
+    pending_request: Optional[Request]
+    """The request presently being sent"""
+    started: bool
+    """Indicate if enabled or not"""
+    max_request_payload_size: int
+    """Maximum size for a request payload gotten from the InfoPoller"""
+    max_response_payload_size: int
+    """Maximum size for a response payload gotten from the InfoPoller"""
+    forbidden_regions: List[MemoryRegion]
+    """List of memory regions to avoid. Gotten from InfoPoller"""
+    readonly_regions: List[MemoryRegion]
+    """List of memory region that can only be read. Gotten from InfoPoller"""
+    memory_write_allowed: bool
+    """Indicates if writing to memory is allowed."""
+    entry_being_updated: Optional[DatastoreEntry]
+    """The datastore entry updated by the actual pending request. None if no request is pending"""
     target_update_request_being_processed: Optional[UpdateTargetRequest]
+    """Update request attached to the entry being updated. It's what's coming from the API"""
     target_update_value_written: Optional[Encodable]
-
+    """The value requested to be written. Has a value when an UpdateTargetRequest is active. ``None`` otherwise"""
     raw_write_request_queue: "queue.Queue[RawMemoryWriteRequest]"
+    """An internal queue that store the raw memory write request (direct memory access not tied to the datastore)"""
     active_raw_write_request: Optional[RawMemoryWriteRequest]
-    active_raw_write_request_remaining_data: Optional[bytes]
+    """The raw write request presently being processed"""
+    active_raw_write_request_remaining_data: Optional[bytearray]
+    """The data of the raw memory write request being processed. Contains the remaining data and gets reduced for each chunk written"""
 
     def __init__(self, protocol: Protocol, dispatcher: RequestDispatcher, datastore: Datastore, request_priority: int):
         self.logger = logging.getLogger(self.__class__.__name__)
@@ -167,7 +197,7 @@ class MemoryWriter:
         self.clear_active_entry_write_request()
         self.clear_active_raw_write_request()
 
-    def clear_config(self) -> None:
+    def _clear_config(self) -> None:
         """Erase the configuration coming from the device handler"""
         self.max_request_payload_size = self.DEFAULT_MAX_REQUEST_PAYLOAD_SIZE
         self.max_response_payload_size = self.DEFAULT_MAX_RESPONSE_PAYLOAD_SIZE
@@ -190,7 +220,17 @@ class MemoryWriter:
     def reset(self) -> None:
         """Put back the memory writer to its startup state"""
         self.set_standby()
-        self.clear_config()
+        self._clear_config()
+
+    def would_send_data(self) -> bool:
+        if not self.started:
+            return False
+        if self.stop_requested:
+            return False
+        if self.pending_request:
+            return False
+
+        return self.datastore.has_pending_target_update() or not self.raw_write_request_queue.empty()
 
     def process(self) -> None:
         """To be called periodically"""
@@ -206,15 +246,15 @@ class MemoryWriter:
             request: Optional[Request] = None
             if self.active_raw_write_request is not None or not self.raw_write_request_queue.empty():
                 assert self.entry_being_updated is None
-                request = self.make_next_raw_memory_write_request()
+                request = self._make_next_raw_memory_write_request()
             elif self.entry_being_updated is not None or self.datastore.has_pending_target_update():
                 assert self.active_raw_write_request is None
-                request = self.make_next_entry_write_request()
+                request = self._make_next_entry_write_request()
 
             if request is not None:
-                self.dispatch(request)
+                self._dispatch(request)
 
-    def make_next_raw_memory_write_request(self) -> Optional[Request]:
+    def _make_next_raw_memory_write_request(self) -> Optional[Request]:
         """Writes a memory block in the device based on a write request tied to no datastore entry. Just a raw write (coming from the API)"""
         request: Optional[Request] = None
 
@@ -243,19 +283,19 @@ class MemoryWriter:
                 self.active_raw_write_request.set_completed(False, "Read only")
                 self.clear_active_raw_write_request()
             else:
-                self.active_raw_write_request_remaining_data = self.active_raw_write_request.data   # bytes are immutable. Makes a copy
+                self.active_raw_write_request_remaining_data = bytearray(self.active_raw_write_request.data)   # bytes are immutable. Makes a copy
 
         if self.active_raw_write_request is not None:
             assert self.active_raw_write_request_remaining_data is not None
             cursor = len(self.active_raw_write_request.data) - len(self.active_raw_write_request_remaining_data)
             n_to_write = min(self.max_request_payload_size, len(self.active_raw_write_request_remaining_data))
-            block = (self.active_raw_write_request.address + cursor, self.active_raw_write_request_remaining_data[:n_to_write])
+            block = (self.active_raw_write_request.address + cursor, bytes(self.active_raw_write_request_remaining_data[:n_to_write]))
             self.active_raw_write_request_remaining_data = self.active_raw_write_request_remaining_data[n_to_write:]
             request = self.protocol.write_memory_blocks([block])
 
         return request
 
-    def make_next_entry_write_request(self) -> Optional[Request]:
+    def _make_next_entry_write_request(self) -> Optional[Request]:
         """
         This method generate a write request by moving in a list of watched variable entries
         It works in a round-robin scheme and will send write request 1 by 1, without agglomeration.
@@ -342,7 +382,7 @@ class MemoryWriter:
 
         return request
 
-    def success_callback(self, request: Request, response: Response, params: Any = None) -> None:
+    def _success_callback(self, request: Request, response: Response, params: Any = None) -> None:
         """Called when a request completes and succeeds"""
         if self.logger.isEnabledFor(logging.DEBUG):  # pragma: no cover
             self.logger.debug("Success callback. Response=%s, Params=%s" % (response, params))
@@ -351,14 +391,14 @@ class MemoryWriter:
         subfn = cmd.MemoryControl.Subfunction(response.subfn)
         if subfn == cmd.MemoryControl.Subfunction.Write or subfn == cmd.MemoryControl.Subfunction.WriteMasked:
             if self.entry_being_updated is not None:
-                self.success_callback_var_write(request, response, params)
+                self._success_callback_var_write(request, response, params)
             elif self.active_raw_write_request is not None:
-                self.success_callback_raw_memory_write(request, response, params)
+                self._success_callback_raw_memory_write(request, response, params)
             else:
                 self.logger.warning("Received a WriteMemory response but none was expected")
         elif subfn == cmd.MemoryControl.Subfunction.WriteRPV:
             if self.entry_being_updated is not None:
-                self.success_callback_rpv_write(request, response, params)
+                self._success_callback_rpv_write(request, response, params)
             else:
                 self.logger.warning("Received a WriteRPV response but none was expected")
         else:
@@ -366,7 +406,7 @@ class MemoryWriter:
 
         self.completed()
 
-    def success_callback_var_write(self, request: Request, response: Response, params: Any = None) -> None:
+    def _success_callback_var_write(self, request: Request, response: Response, params: Any = None) -> None:
         """Called when a memory write request completes and succeeds"""
         assert self.entry_being_updated is not None
         assert self.target_update_request_being_processed is not None
@@ -398,7 +438,7 @@ class MemoryWriter:
 
         self.clear_active_entry_write_request()
 
-    def success_callback_raw_memory_write(self, request: Request, response: Response, params: Any = None) -> None:
+    def _success_callback_raw_memory_write(self, request: Request, response: Response, params: Any = None) -> None:
         assert self.active_raw_write_request is not None
         assert self.active_raw_write_request_remaining_data is not None
 
@@ -431,7 +471,7 @@ class MemoryWriter:
             self.active_raw_write_request.set_completed(success=False, failure_reason="Refused by the device")
             self.clear_active_raw_write_request()
 
-    def success_callback_rpv_write(self, request: Request, response: Response, params: Any = None) -> None:
+    def _success_callback_rpv_write(self, request: Request, response: Response, params: Any = None) -> None:
         """Called when a RPV write request completes and succeeds"""
         assert self.entry_being_updated is not None
         assert self.target_update_request_being_processed is not None
@@ -459,7 +499,7 @@ class MemoryWriter:
 
         self.clear_active_entry_write_request()
 
-    def failure_callback(self, request: Request, params: Any = None) -> None:
+    def _failure_callback(self, request: Request, params: Any = None) -> None:
         """Callback called by the request dispatcher when a request fails to complete"""
         if self.logger.isEnabledFor(logging.DEBUG):  # pragma: no cover
             self.logger.debug("Failure callback. Request=%s. Params=%s" % (request, params))
@@ -486,14 +526,14 @@ class MemoryWriter:
         """Common code after success and failure callbacks"""
         self.pending_request = None
 
-    def dispatch(self, request: Request) -> None:
+    def _dispatch(self, request: Request) -> None:
         """Sends a request to the request dispatcher and assign the corrects completion callbacks"""
         if self.logger.isEnabledFor(logging.DEBUG):  # pragma: no cover
             self.logger.debug('Registering a MemoryWrite request. %s' % (request))
         self.dispatcher.register_request(
             request=request,
-            success_callback=self.success_callback,
-            failure_callback=self.failure_callback,
+            success_callback=self._success_callback,
+            failure_callback=self._failure_callback,
             priority=self.request_priority
         )
         self.pending_request = request
